"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const medusa_1 = require("@medusajs/medusa");
const axios_1 = __importDefault(require("axios"));
const format_registration_name_1 = __importDefault(require("@medusajs/medusa/dist/utils/format-registration-name"));
const types_1 = require("../types");
const utils_1 = require("../utils");
class SentryService extends medusa_1.TransactionBaseService {
    constructor({ manager, eventBusService }, config) {
        // @ts-ignore
        super(...arguments);
        this.sentryApiBaseUrl = 'https://sentry.io/api/0/organizations';
        this.manager_ = manager;
        this.config_ = config;
        this.eventBusService_ = eventBusService;
    }
    /**
     * Fetch paginated transactions from an organisation project on sentry
     * @param organisation The organisation on which to fetch the transactions
     * @param project The project in the organisation on which to fetch the transactions
     * @param query Equation to filter the result (https://docs.sentry.io/product/sentry-basics/search/)
     * @param statsPeriod The period from when to fetch the transactions (default: 24h)
     * @param perPage The number of transaction per page
     * @param token The token to use to send request to sentry
     * @param cursor The cursor to send to fetch the transactions for a given page
     * @return The result is composed of the data and the next cursor for the pagination purpose
     */
    async fetchTransactions({ organisation, project, query, statsPeriod, perPage, token, cursor, }) {
        perPage = Math.min(Number(perPage !== null && perPage !== void 0 ? perPage : 100), 100);
        const queryParams = {
            field: ['transaction', 'tpm()', 'p50()', 'p75()', 'p95()', 'failure_rate', 'apdex()'],
            per_page: perPage,
            project,
            query: `event.type:transaction${query ? ' AND ' + query : ''}`,
            statsPeriod,
            sort: '-transaction',
            // The three values from cursor are: cursor identifier (integer, usually 0), row offset, and is_prev (1 or 0).
            // e.g 0:10:0
            cursor,
        };
        return await this.fetchSentryData({
            organisation,
            token,
            perPage,
            queryParams,
        });
    }
    /**
     * Fetch paginated transaction events from an organisation project on sentry
     * @param transaction The transaction for which to fetch the events (e.g "GET /admin/users")
     * @param organisation The organisation on which to fetch the transactions
     * @param project The project in the organisation on which to fetch the transactions
     * @param query Equation to filter the result (https://docs.sentry.io/product/sentry-basics/search/)
     * @param statsPeriod The period from when to fetch the transactions (default: 24h)
     * @param perPage The number of transaction per page
     * @param token The token to use to send request to sentry
     * @param cursor The cursor to send to fetch the transactions for a given page
     * @return The result is composed of the data and the next cursor for the pagination purpose
     */
    async fetchTransactionEvents({ transaction, organisation, project, query, statsPeriod, perPage, token, cursor, }) {
        perPage = Math.min(Number(perPage !== null && perPage !== void 0 ? perPage : 100), 100);
        const queryParams = {
            field: ['id', 'transaction.duration', 'timestamp', 'spans.db', 'project'],
            per_page: perPage,
            project,
            query: `event.type:transaction AND transaction:"${transaction}"${query ? ' AND ' + query : ''}`,
            statsPeriod,
            sort: '-timestamp',
            // The three values from cursor are: cursor identifier (integer, usually 0), row offset, and is_prev (1 or 0).
            // e.g 0:10:0
            cursor,
        };
        return await this.fetchSentryData({
            organisation,
            token,
            perPage,
            queryParams,
        });
    }
    /**
     * Fetch paginated transaction events from an organisation project on sentry
     * @param transaction The transaction for which to fetch the stats if any trasaction is provided (e.g "GET /admin/users")
     * @param organisation The organisation on which to fetch the transactions
     * @param statsPeriod The period from when to fetch the transactions (default: 24h)
     * @param project The project in the organisation on which to fetch the transactions
     * @param token The token to use to send request to sentry
     * @return The result is composed of the data and the next cursor for the pagination purpose
     */
    async fetchTransactionsStats({ transaction, organisation, project, statsPeriod, token, }) {
        const queryParams = {
            interval: '1h',
            partial: 1,
            project,
            statsPeriod,
            query: `event.type:transaction ${transaction ? `transaction:"${transaction}"` : ''}`,
            yAxis: ['apdex()', 'tpm()', 'failure_rate()'],
        };
        return await this.fetchSentryStats({
            organisation,
            token,
            queryParams,
        });
    }
    async handleIssues(data) {
        await this.manager_.transaction(async (transactionManager) => {
            if ((0, utils_1.isFunction)(this.config_.webHookOptions.emitOnIssue)) {
                return await this.config_.webHookOptions.emitOnIssue(this.__container__, data);
            }
            await this.eventBusService_
                .withTransaction(transactionManager)
                .emit(types_1.SentryWebHookEvent.SENTRY_RECEIVED_ISSUE, data);
        });
    }
    async handleErrors(data) {
        await this.manager_.transaction(async (transactionManager) => {
            if ((0, utils_1.isFunction)(this.config_.webHookOptions.emitOnError)) {
                return await this.config_.webHookOptions.emitOnError(this.__container__, data);
            }
            await this.eventBusService_
                .withTransaction(transactionManager)
                .emit(types_1.SentryWebHookEvent.SENTRY_RECEIVED_ERROR, data);
        });
    }
    async handleComments(data) {
        await this.manager_.transaction(async (transactionManager) => {
            if ((0, utils_1.isFunction)(this.config_.webHookOptions.emitOnComment)) {
                return await this.config_.webHookOptions.emitOnComment(this.__container__, data);
            }
            await this.eventBusService_
                .withTransaction(transactionManager)
                .emit(types_1.SentryWebHookEvent.SENTRY_RECEIVED_COMMENT, data);
        });
    }
    async handleAlerts(data) {
        await this.manager_.transaction(async (transactionManager) => {
            if ((0, utils_1.isFunction)(this.config_.webHookOptions.emitOnEventOrMetricAlert)) {
                return await this.config_.webHookOptions.emitOnEventOrMetricAlert(this.__container__, data);
            }
            await this.eventBusService_
                .withTransaction(transactionManager)
                .emit(types_1.SentryWebHookEvent.SENTRY_RECEIVED_EVENT_OR_METRIC_ALERT, data);
        });
    }
    async handleInstallation(data) {
        await this.manager_.transaction(async (transactionManager) => {
            if ((0, utils_1.isFunction)(this.config_.webHookOptions.emitOnInstallOrDeleted)) {
                return await this.config_.webHookOptions.emitOnInstallOrDeleted(this.__container__, data);
            }
            await this.eventBusService_
                .withTransaction(transactionManager)
                .emit(types_1.SentryWebHookEvent.SENTRY_RECEIVED_INSTALL_OR_DELETED, data);
        });
    }
    async fetchSentry({ organisation, token, queryParams, customTargetPathSegment, }) {
        const url = this.sentryApiBaseUrl + `/${organisation}/${customTargetPathSegment ? customTargetPathSegment : 'events/'}`;
        const searchParams = new URLSearchParams();
        Object.entries(queryParams).forEach(([key, value]) => {
            if (Array.isArray(value)) {
                value.forEach((val) => {
                    searchParams.append(key, val);
                });
            }
            else {
                value && searchParams.append(key, value === null || value === void 0 ? void 0 : value.toString());
            }
        });
        const { data, headers } = await axios_1.default.get(url, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
            params: searchParams,
        });
        return { data, headers };
    }
    async fetchSentryData({ organisation, token, queryParams, perPage, }) {
        const { data: { data, meta }, headers, } = await this.fetchSentry({
            organisation,
            token,
            queryParams,
        });
        const nextCursor = SentryService.buildNextCursor(headers['link']);
        const currentCursor = SentryService.buildCurrentCursor(headers['link'], perPage);
        let prevCursor;
        if (currentCursor) {
            prevCursor = SentryService.buildPrevCursor(currentCursor, perPage);
        }
        return { data, meta, prev_cursor: prevCursor, next_cursor: nextCursor };
    }
    async fetchSentryStats({ organisation, token, queryParams, }) {
        const { data } = await this.fetchSentry({
            organisation,
            token,
            queryParams,
            customTargetPathSegment: 'events-stats/',
        });
        return data;
    }
    static buildNextCursor(link) {
        var _a;
        let result;
        const nextCursorMatch = link === null || link === void 0 ? void 0 : link.match(/.*cursor="(.*)"$/);
        const hasResults = link === null || link === void 0 ? void 0 : link.match(/.*results="(.*)";/);
        if (hasResults && hasResults[1] === 'true') {
            if (nextCursorMatch && nextCursorMatch[1]) {
                result = (_a = nextCursorMatch[1]) === null || _a === void 0 ? void 0 : _a.split(',')[0];
            }
        }
        return result;
    }
    static buildCurrentCursor(link, perPage) {
        var _a;
        let result;
        const nextCursorMatch = link === null || link === void 0 ? void 0 : link.match(/.*cursor="(.*)"$/);
        if (nextCursorMatch && nextCursorMatch[1]) {
            result = (_a = nextCursorMatch[1]) === null || _a === void 0 ? void 0 : _a.split(',')[0];
        }
        return result ? this.buildPrevCursor(result, perPage) : undefined;
    }
    static buildPrevCursor(nextCursor, perPage) {
        const parts = nextCursor.split(':');
        const prevCursorItems = Number(parts[1]) - perPage;
        if (prevCursorItems < 0) {
            return;
        }
        return `${parts[0]}:${prevCursorItems}:${parts[2]}`;
    }
}
exports.default = SentryService;
SentryService.RESOLVE_KEY = (0, format_registration_name_1.default)(`${process.cwd()}/services/sentry.js`);
//# sourceMappingURL=sentry.js.map